@@ -5,7 +5,7 @@ use POSIX qw(strftime);
 use File::Spec;
 use File::MimeInfo;
 
-our $VERSION = 0.03;
+our $VERSION = 0.04;
 
 =head1 NAME
 
@@ -56,13 +56,27 @@ Simple constructor. DIR is the root directory of the repository.
 =cut
 
 sub new {
-	my ($class, $dir) = @_;
+	my ($class, $namespace, $dir) = @_;
 	$dir = File::Spec->canonpath($dir);
-	my $self = bless {root => $dir}, $class;
+	my $self = bless {
+		namespace => $namespace,
+		root      => $dir
+	}, $class;
 	
-	# FIXME algo below is unwanted
-	# we need some advandced caching for all this
-	#$self->_scan_for_plugins($self->{root});
+	my $conf = File::Spec->catfile($dir, '.zim.config');
+	if (-f $conf and -r _) {
+		$self->{conffile} = $conf;
+		for (split /\n/, Zim::Page::File::_read_file($conf)) {
+			/^:(\S+?)=(\S+)/ or next;
+			my ($subspace, $mod) = ($1, $2);
+			$subspace =~ s/:+$//;
+			eval "use $mod;";
+			die if $@;
+			my $dir = $self->filename($subspace, 'DIR');
+			$self->{plugins}{$subspace} = $mod->new($subspace, $dir);
+			#print "plug: $mod for $namespace\n";
+		}
+	}
 
 	return $self;
 }
@@ -76,7 +90,14 @@ in the listing.
 
 sub list_pages {
 	my ($self, $namespace) = @_;
+	
+	$namespace =~ s/:*$/:/;
+	if (my $plug = $self->belongs_to_plugin($namespace)) {
+		return $self->{plugins}{$plug}->list_pages($namespace);
+	}
+
 	my $dir = $self->filename($namespace, 'DIR');
+	#print "list pages $namespace => $dir\n";
 	return () unless -d $dir;
 	opendir DIR, $dir or die "Could not list dir $dir\n";
 	my @items = grep {defined $_} map {
@@ -92,6 +113,14 @@ sub list_pages {
 	return sort {lc($a) cmp lc($b)} @items;
 }
 
+sub belongs_to_plugin {
+        my ($self, $page) = @_;
+	$page =~ s/^:*$self->{namespace}:*//;
+        my ($plug) = grep {$page =~ /^$_:/} keys %{$self->{plugins}};
+        return $plug;
+}
+
+
 =item page_exists(PAGE_NAME)
 
 Returns TRUE when there is an existing page under this name.
@@ -99,14 +128,13 @@ Returns TRUE when there is an existing page under this name.
 =cut
 
 sub page_exists {
-	my ($self, $page) = @_;
+	my ($self, $name) = @_;
 	
-#	if (my $plug = $self->belongs_to_plugin($page)) {
-#		$page =~ s/$plug://;
-#		return $self->{plugins}{$plug}->page_exists($page);
-#	}
+	if (my $plug = $self->belongs_to_plugin($name)) {
+		return $self->{plugins}{$plug}->page_exists($name);
+	}
 	
-	return -f $self->filename($page);
+	return -f $self->filename($name);
 }
 
 =item load_page(PAGE_NAME)
@@ -118,10 +146,9 @@ Returns an object of the type Zim::Page.
 sub load_page {
 	my ($self, $name) = @_;
 
-#	if (my $plug = $self->belongs_to_plugin($page)) {
-#		$page =~ s/$plug://;
-#		return $self->{plugins}{$plug}->load_page($page);
-#	}
+	if (my $plug = $self->belongs_to_plugin($name)) {
+		return $self->{plugins}{$plug}->load_page($name);
+	}
 	
 	my $page = Zim::Page::File->new($self, $name);
 	#print "page $page => file ".$page->filename."\n";
@@ -229,7 +256,8 @@ Gives the filename corresponding to a page name.
 NAME is a string. (This lookup is more difficult then it might seem.)
 
 DIR is a boolean that can be set to tell that NAME is expected to
-be a directory.
+be a directory. This behaviour is also triggered if the page name
+ends with a ':'.
 
 In list context filename, directory and real page name are returned.
 
@@ -238,6 +266,9 @@ In list context filename, directory and real page name are returned.
 sub filename {
 	my ($self, $page, $is_dir) = @_;
 
+	$page =~ s/^:*$self->{namespace}:*//;
+	$is_dir++ if $page =~ s/:+$//;
+	
 	my @dirs = split /:+/, $page;
 	my $basename = pop @dirs unless $is_dir;
 
@@ -260,8 +291,11 @@ sub filename {
 		$dir = File::Spec->catdir($self->{root}, @dirs);
 	}
 
-	return wantarray ? ($dir, $dir, join ':', @realname) : $dir
-		if $is_dir;
+	if ($is_dir) {
+		return $dir unless wantarray;
+		my $realname = join ':', $self->{namespace}, @realname;
+		return $dir, $dir, $realname;
+	}
 
 	# check which filename to use
 	my $file;
@@ -275,7 +309,9 @@ sub filename {
 	push @realname, $basename;
 	#print "$page => file: $file, realname: @realname\n";
 
-	return wantarray ? ($file, $dir, join ':', @realname) : $file;
+	return $file unless wantarray;
+	my $realname = join ':', $self->{namespace}, @realname;
+	return $file, $dir, $realname;
 }
 
 =item pagename(FILE)
@@ -290,11 +326,13 @@ sub pagename {
 	# $file =~ s/^\Q$self->{root}\E//;
 	my @parts = map  {s/_files$//; $_} File::Spec->splitdir($file);
 	$parts[-1] =~ s/\.txt$//;
-	return join ':', @parts;
+	return join ':', $self->{namespace}, @parts;
 }
 
 package Zim::Formatter;
 
+no warnings;
+
 =back
 
 =head2 Zim::Formatter
@@ -320,12 +358,13 @@ sub text2blocks {
 	# empty lines disappear
 	my ($self, @text) = @_;
 	return grep {defined $_} map {
-		(! /\S/)        ?  undef                         :
-		/^(\t|\s\s)/    ? ($self->parse_pre($_), "\n"  ) :
+		#print STDERR "#>>$_<<\n";
+		(! /\S/)        ? $_                            :
+		/^(\t|\s\s)/    ? ($self->parse_pre($_),  ) :
 		/\n?^==+\s+\S+/ ? ($self->parse_head($_)       ) :
-		                  ($self->parse_block($_), "\n") ;
+		                  ($self->parse_block($_), ) ;
 	} map {
-		split /(?:^\s*\n)+|(\n?^==+\s+\S.*\n(\s*\n)?)/m,
+		split /((?:^\s*\n)+|\n?^==+\s+\S.*\n(?:\s*\n)?)/m,
 	} @text;
 }
 
@@ -358,7 +397,7 @@ sub parse_image {
 	my ($self, $text) = @_;
 	my $i = 0;
 	return map {
-		($i++ % 2) ? ['image', $_] : $_
+		($i++ % 2) ? [image => $_] : $_
 	} split /\{\{(.+?)\}\}/, $text;
 }
 
@@ -368,12 +407,19 @@ sub parse_link { # parse links and urls
 	return map {
 		if ($i++ % 2) {
 			if (s/^\[\[(.+)\]\]$/$1/) {
-				if (/^(.*)\|(.*)$/) { ['link', $2, $1] }
-				else                { ['link', $_]     }
+				if (/^(.*)\|(.*)$/) { [link => $2, $1] }
+				else                { [link => $_]     }
 			}
-			else { ['link', $_] }
+			elsif (/^\w+:\/\//)  { [link => $_ ]             }
+			elsif (s/^mailto://) { [link => $_, "mailto:$_"] }
+			elsif (/@/)	     { [link => $_, "mailto:$_"] }
+			else                 { [link => $_ ]             }
 		} else { $_ }
-	} split /(\[\[.+?\]\]|\b\w+:\/\/\S+[^\s\,\.\;])/, $text;
+	} split /(
+		\[\[.+?\]\]                |
+		\b\w+:\/\/\S+[^\s\,\.\;\)]   |
+		\b(?:mailto:)?\S+@\S+\.\w+
+	)/x, $text;
 }
 
 sub parse_styles { # parse blocks of bold, italic and underline
@@ -381,10 +427,10 @@ sub parse_styles { # parse blocks of bold, italic and underline
 	my $i = 0;
 	return	map {
 		unless ($i++ % 2) { $_ }
-		elsif (/^\'\'(.+)\'\'$/) { ['pre',       $1] }
-		elsif (/^\*\*(.+)\*\*$/) { ['bold',      $1] }
-		elsif (/^\/\/(.+)\/\/$/) { ['italic',    $1] }
-		elsif (/^\_\_(.+)\_\_$/) { ['underline', $1] }
+		elsif (/^\'\'(.+)\'\'$/) { [pre       => $1] }
+		elsif (/^\*\*(.+)\*\*$/) { [bold      => $1] }
+		elsif (/^\/\/(.+)\/\/$/) { [italic    => $1] }
+		elsif (/^\_\_(.+)\_\_$/) { [underline => $1] }
 	} split /(?<!\S)(
 		\'\'.+?\'\' |
 		\*\*.+?\*\* |
@@ -410,9 +456,12 @@ sub blocks2text { ## unknown tags just fall through as plain text
 		}
 		my ($tag, @node) = @{ shift @blocks };
 		if ($tag eq 'link') { 
-			@node = (@node > 1) ? ("[[$node[1]|$node[0]]]") :
-				($node[0] =~ /^\w+:\/\//) ? ($node[0])  :
-				                       ("[[$node[0]]]") ;
+			@node = (@node > 1) ? (
+					($node[1] =~ s/^mailto://)
+						? $node[1]
+						: "[[$node[1]|$node[0]]]" ) : 
+				($node[0] =~ /^\w+:\/\//)
+					? ($node[0]) : ("[[$node[0]]]") ;
 		}
 		elsif ($tag eq 'pre')  {
 			for (@node) {
@@ -854,15 +903,15 @@ Text:
 
 Parses to:
 
-	['head1', 'NAME'],
+	[head1 => 'NAME'],
 	"\n\nFoo - supplement to Bar\n\n",
-	['head1', 'SYNOPSIS'],
+	[head1 => 'SYNOPSIS'],
 	"\n\n",
-	['pre', "use Foo;\nFoo->bar;\n"], # indenting removed
+	[pre   => "use Foo;\nFoo->bar;\n"], # indenting removed
 	"\n",
-	['head1', 'SEE ALSO'],
+	[head1 => 'SEE ALSO'],
 	"\n\n",
-	['link', 'Bar', 'Bar'],  # link text and link target
+	[link  => 'Bar', 'Bar'],  # link text and link target
 	"\n"
 
 FIXME a more complicated structure may be needed when we want to
